/**********************************************************************************************
    Copyright (C) 2007 Oliver Eichler oliver.eichler@gmx.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

**********************************************************************************************/

#include "GeoMath.h"
#include <stdlib.h>
#include <QVector>


//Copyright (C) 1999 Alan Bleasby
void GPS_Math_Deg_To_DegMin(double v, int32_t *d, double *m)
{
    int32_t sign;

    if(v<(double)0.)
    {
        v *= (double)-1.;
        sign = 1;
    }
    else{
        sign = 0;
    }

    *d = (int32_t)v;
    *m = (v-(double)*d) * (double)60.0;
    if(*m>(double)59.999)
    {
        ++*d;
        *m = (double)0.0;
    }

    if(sign){
        *d = -*d;
    }

    return;
}

void GPS_Math_DegMin_To_Deg(const int32_t d, const double m, double& deg)
{

    deg = ((double)abs(d)) + m / (double)60.0;
    if(d<0){
        deg = -deg;
    }

    return;
}

/// calc if 2 line segments intersect
/**
    The 1st line is defined by (x11,y11) - (x12,y12)
    The 2nd line is defined by (x21,y21) - (x22,y22)

*/
bool testLineSegForIntersect(float x11, float y11, float x12, float y12, float x21, float y21, float x22, float y22)
{
/*
        float denom = ((other_line.end_.y_ - other_line.begin_.y_)*(end_.x_ - begin_.x_)) -
                      ((other_line.end_.x_ - other_line.begin_.x_)*(end_.y_ - begin_.y_));

        float nume_a = ((other_line.end_.x_ - other_line.begin_.x_)*(begin_.y_ - other_line.begin_.y_)) -
                       ((other_line.end_.y_ - other_line.begin_.y_)*(begin_.x_ - other_line.begin_.x_));

        float nume_b = ((end_.x_ - begin_.x_)*(begin_.y_ - other_line.begin_.y_)) -
                       ((end_.y_ - begin_.y_)*(begin_.x_ - other_line.begin_.x_));
*/
    float denom  = ((y22 - y21) * (x12 - x11)) - ((x22 - x21) * (y12 - y11));
    float nume_a = ((x22 - x21) * (y11 - y21)) - ((y22 - y21) * (x11 - x21));
    float nume_b = ((x12 - x11) * (y11 - y21)) - ((y12 - y11) * (x11 - x21));

    if(denom == 0.0f) return false;

    float ua = nume_a / denom;
    float ub = nume_b / denom;

    if(ua >= 0.0f && ua <= 1.0f && ub >= 0.0f && ub <= 1.0f){
        return true;
    }
    return false;
}

bool testPointInPolygon(const XY& pt, const QVector<XY>& poly1)
{

    bool    c = false;
    int     npol;
    int     i = 0, j = 0;
    XY      p1, p2;          // the two points of the polyline close to pt
    double  x = pt.u;
    double  y = pt.v;


    npol = poly1.count();
    if(npol > 2){

        // see http://local.wasp.uwa.edu.au/~pbourke/geometry/insidepoly/
        for (i = 0, j = npol-1; i < npol; j = i++) {
            p1 = poly1[j];
            p2 = poly1[i];

            if ((((p2.v <= y) && (y < p1.v))  || ((p1.v <= y) && (y < p2.v))) &&
                (x < (p1.u - p2.u) * (y - p2.v) / (p1.v - p2.v) + p2.u))
            {
                c = !c;
            }
        }
    }
    return c;
}

bool testPolygonsForIntersect(const QVector<XY>& poly1, const QVector<XY>& poly2)
{

    int n;
    int npol1 = poly1.count();
    int npol2 = poly2.count();

    if(npol1 < 2 || npol2 < 2) return false;


    // test if points of poly1 are within poly2.
    for(n = 0; n < npol1; ++n){
        if(testPointInPolygon(poly1[n],poly2)) return true;
    }


    // test if lines of poly1 intersect with lines from poly2
    int i1 = 0, j1 = 0;
    int i2 = 0, j2 = 0;

    XY  p1, p2, p3, p4;

    for (i1 = 0, j1 = npol1-1; i1 < npol1; j1 = i1++) {
        p1 = poly1[j1];
        p2 = poly1[i1];
        for (i2 = 0, j2 = npol2-1; i2 < npol2; j2 = i2++) {
            p3 = poly2[j2];
            p4 = poly2[i2];
            if(testLineSegForIntersect(p1.u,p1.v,p2.u,p2.v,p3.u,p3.v,p4.u,p4.v)){
                return true;
            }
        }
    }
    return false;
}


